import '../references/extensions.dist.js';

import './project.scss';


class ProjectPlanner {
  constructor(el) {
    const a = this;

    a.Container = el;
    a.Columns = [
      null,
      null,
      "Task Name",
      "Duration",
      "Start",
      "Finish",
      "Predecessor",
      "Resource Names",
      null
    ];
    a.Tasks = [];

		a.#initialiseComponents();
  }

  #initialiseComponents() {
    const a = this;

    if (!a.Container.classList.contains("literyzjs-project")) {
      a.Container.classList.add("literyzjs-project");
    }

    let htmlContent = "";
    htmlContent += "<table class='project-planner'>";
    htmlContent += a.#renderTHead();
    htmlContent += "<tbody>";
    htmlContent += a.#renderPlaceholder();
    htmlContent += "</tbody>";
    htmlContent += "</table>";

    a.Container.innerHTML = htmlContent;
  }


  Clear() {
    const a = this;

    a.Load([]);
  }

  Load(tasks) {
    const a = this;

    a.Tasks = tasks;

    if (!a.Container.classList.contains("literyzjs-project")) {
      a.Container.classList.add("literyzjs-project");
    }

    let htmlContent = "";
    htmlContent += "<table class='project-planner'>";
    htmlContent += a.#renderTHead();
    htmlContent += "<tbody>";

    tasks.forEach(e => {
      htmlContent += a.#renderRow(e);
    });

    htmlContent += "</tbody>";
    htmlContent += "</table>";

    a.Container.innerHTML = htmlContent;
  }


  #renderTHead() {
    const a = this;

    let htmlContent = "";
    htmlContent += "<thead>";
    htmlContent += "<tr>";

    a.Columns.forEach(e => {
      htmlContent += "<th>" + (e ?? "") + "</th>";
    });

    htmlContent += "</tr>";
    htmlContent += "</thead>";

    return htmlContent;
  }

  #renderPlaceholder() {
    const a = this;

    let htmlContent = "";
    htmlContent += "<tr>";
    htmlContent += "<td></td>";
    htmlContent += "<td colspan='" + (a.Columns.length - 1) + "' class='c'>Loading...</td>";
    htmlContent += "</tr>";

    return htmlContent;
  }

  #renderRow(task) {
    const a = this;

    let htmlContent = "";

    if (task.IsCollated == true) {
      htmlContent += "<tr class='b'>";
    } else {
      htmlContent += "<tr>";
    }

    htmlContent += "<td class='c'>" + task.Order + "</td>";
    htmlContent += "<td></td>";
    htmlContent += "<td>";

    for (let i=0; i<task.Level; i++) {
      htmlContent += "<span class='i'></span>";
    }

    htmlContent += "<input type='text' value='" + task.Name + "'>";
    htmlContent += "</td>";
    htmlContent += "<td>" + task.Duration + " day" + (parseInt(task.Duration) == 1 ? "" : "s") + "</td>";
    htmlContent += "<td class='c'>" + new Date(task.StartDate).toLocaleDateString() + "</td>";
    htmlContent += "<td class='c'>" + new Date(task.FinishDate).toLocaleDateString() + "</td>";
    htmlContent += "<td class='c'>" + a.#findTaskNo(task.PredecessorTaskID) + "</td>";
    htmlContent += "<td></td>";
    htmlContent += "<td></td>";
    htmlContent += "</tr>";

    return htmlContent;
  }

  #findTaskNo(id) {
    const a = this;

    if (id == null) {
      return "";
    }

    const item = a.Tasks.first("ID", id);

    return (item == null) ? "" : item.Order;
  }

}


export default ProjectPlanner;